// ===============================================================
// == Model Repair for Knuth's model of a fair die (PRISM/PARAM)==
// ==                                                           ==
// ==   Supplement of the work:                                 ==
// ==                                                           ==
// ==     Model Repair for probabilistic systems                ==
// ==     E. Bartocci, R. Grosu, P. Katsaros,                   ==
// ==     C.R. Ramakrishnan, Scott A. Smolka, TACAS 2011        ==
// ==                                                           ==
// ==   Authors of this file:                                   ==
// ==                                                           ==
// ==     E. Bartocci, P. Katsaros                              ==
// ==                                                           ==
// ==   Date:  07/07/10                                         ==
// ==                                                           ==
// ==                                                           ==
// ==   SUNY Stony Brook, Stony Brook, NY                       ==
// ==   Aristotle University of Thessaloniki, Greece            ==
// ==                                                           ==
// ===============================================================
// Knuth's model of a fair die using only fair coins
dtmc

//Uncomment the next three lines to have the parametric version 
//param double v1;
//param double v2;
//param double v3;

//Uncomment the next 3 lines to see the original
//const double v1 =   0;
//const double v2 =   0;
//const double v3 =   0;

//Uncomment the next 3 lines the model repair P<=1/8 F[s=7 & d=1] with 
//cost function min v1^2 + v2^2 + v3^2    Figure 3 a
//const double v1 =  -0.042;
//const double v2 =  -0.056;
//const double v3 =   0.030;

//Uncomment the next 3 lines the model repair P<=1/8 F[s=7 & d=1] with 
//cost function min 10 * v1^2 + 5 * v2^2 + v3^2    Figure 3 b
const double v1 =  -0.012;
const double v2 =  -0.034;
const double v3 =   0.106;

module die

	// local state
	s : [0..7] init 0;
	// value of the dice
	d : [0..6] init 0;
	
	[] s=0 -> 0.5 + v1  : (s'=1) + 0.5 - v1 : (s'=2);
	[] s=1 -> 0.5 + v2  : (s'=3) + 0.5 - v2 : (s'=4);
	[] s=2 -> 0.5 + v2  : (s'=5) + 0.5 - v2 : (s'=6);
	[] s=3 -> 0.5 + v3  : (s'=1) + 0.5 - v3: (s'=7) & (d'=1);
	[] s=4 -> 0.5 + v3  : (s'=7) & (d'=2) + 0.5 - v3: (s'=7) & (d'=3);
	[] s=5 -> 0.5 + v3  : (s'=7) & (d'=4) + 0.5 - v3: (s'=7) & (d'=5);
	[] s=6 -> 0.5 + v3  : (s'=2) + 0.5 -v3 : (s'=7) & (d'=6);
	[] s=7 -> (s'=7);
	
endmodule

